/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	dcf.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

 #include "asf.h"
 #include "dcf.h"
 #include "timer.h"
 #include "lcd.h"
 #include "main.h"
 #include "uart_menu.h"
 #include "string.h"

 char myTime[16];		// String zum Erzeugen der Displayanzeige - Uhrzeit
 char myDate[16];		// String zum Erzeugen der Displayanzeige - Datum und Wochentag

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		dcf_init:	Initialisiert des DCF-Moduls.
								Diese Methode aktiviert den externen Interrupt fr den DATA-Pin des DCF-Moduls
								verbunden mit Pin PIO_PA4. Des Weiteren werden alle Merker-Variablen zurckgesetzt,
								der Timer TC2 konfiguriert und gestartet und auf dem Display "wait for DCF..."
								angezeigt.
 -------------------------------------------------------------------------------------------------------------------- */
 void dcf_init(void)
 {
	pmc_enable_periph_clk(DCF_DATA_ID);
	NVIC_EnableIRQ(PIOA_IRQn);

	dcf_invalidSignal = false;
	dcf_synced = false;
	dcf_signalTestProcess = false;
	dcf_signalTestDetection = true;
	dcf_messageDelayCounter = 5;
	dcf_signalTestCounter = 0;
	dcf_validSignalCounter = 0;
	dcf_syncCounter = 0;
	dcf_bitCounter = 0;
	dcf_errorDetection = 0;
	dcf_startup = 0;	
	dcf_completed = false;
	dcf_invalidSync = false;

	tc_config_TC2(TIMER_FREQUENZ_2);
	LCD_Clear();
	LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "wait for DCF...");
	LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
	lcd_setCursor(0, LINE4);
	Write_char_lcd(LCD_MENU_CURSOR);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		dcf_stop:	Deaktivieren des DCF-Moduls.
								Alle Speicher-Variablen werden zurckgesetzt, anschlieend der Timer TC2 gestoppt,
								der Externe Interrupt fr den DATA-Pin des DCF-Moduls deaktiviert und der PON-Pin
								des DCF-Moduls auf High-Pegel gebracht, um das Modul abzuschalten.
 -------------------------------------------------------------------------------------------------------------------- */
 void dcf_stop(void)
 {
	dcf_receivedData_min = 0;
	dcf_receivedData_h = 0;
	dcf_receivedData_weekday = 0;
	dcf_receivedData_date_day = 0;
	dcf_receivedData_date_month = 0;
	dcf_receivedData_date_year = 0;
	dcf_validSignalCounter = 0;
	dcf_signalTestCounter = 0;
	dcf_validTimeReceived = false;
	dcf_validDateReceived = false;
	
	tc_stop(TC0,CHANNEL_TIMER_2);
	NVIC_DisableIRQ(TC2_IRQn);
	gpio_set_pin_high(DCF_PON_GPIO);
	//TEST
	gpio_set_pin_low(LED1);
	gpio_set_pin_low(LED2);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		updateTime:		Aktualisieren der Uhrzeit des RTC.
									Speichert die empfangenen Daten fr Stunden und Minuten jeweils in einen
									Zwischenspeicher und anschlieend in den Speicher des RTC. Anschlieend werden
									die Variablen zum Speichern der empfangenen Daten zurckgesetzt, um Fehler
									zu vermeiden.
 -------------------------------------------------------------------------------------------------------------------- */
 void updateTime(void)
 {
	uint32_t dcf_time_min = 0;
	uint32_t dcf_time_h = 0;
	dcf_time_min = ((dcf_receivedData_min & DCF_MIN_UNITS_MASK) + (((dcf_receivedData_min & DCF_MIN_TENS_MASK)>>4)*10));
	dcf_time_h = ((dcf_receivedData_h & DCF_H_UNITS_MASK) + (((dcf_receivedData_h & DCF_H_TENS_MASK)>>4)*10));

	//write time to RTC
	rtc_set_time(RTC, dcf_time_h, dcf_time_min, 0);

	dcf_receivedData_min = 0;
	dcf_receivedData_h = 0;
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		updateDate:		Aktualisieren des Datums des RTC.
									Speichert die empfangenen Daten fr Wochentag, Tag, Monat und Jahr jeweils in einen
									Zwischenspeicher und anschlieend in den Speicher des RTC. Anschlieend werden
									die Variablen zum Speichern der empfangenen Daten zurckgesetzt, um Fehler
									zu vermeiden.
 -------------------------------------------------------------------------------------------------------------------- */
 void updateDate(void)
 {
	 uint32_t dcf_date_weekday= 0;
	 uint32_t dcf_date_day = 0;
	 uint32_t dcf_date_month = 0;
	 uint32_t dcf_date_year = 0;
	 dcf_date_weekday = dcf_receivedData_weekday;
	 dcf_date_day = ((dcf_receivedData_date_day & DCF_DAY_UNITS_MASK) + (((dcf_receivedData_date_day & DCF_DAY_TENS_MASK)>>4)*10));
	 dcf_date_month = ((dcf_receivedData_date_month & DCF_MONTH_UNITS_MASK) + (((dcf_receivedData_date_month & DCF_MONTH_TENS_MASK)>>4)*10));
	 dcf_date_year = ((dcf_receivedData_date_year & DCF_YEAR_UNITS_MASK) + (((dcf_receivedData_date_year & DCF_YEAR_TENS_MASK)>>4)*10));

	 // write date to RTC
	 rtc_set_date(RTC, dcf_date_year+2000, dcf_date_month, dcf_date_day, dcf_date_weekday);

	 dcf_receivedData_weekday = 0;
	 dcf_receivedData_date_day = 0;
	 dcf_receivedData_date_month = 0;
	 dcf_receivedData_date_year = 0;
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		checkParity:	Parittsprfung der empfangenen Daten.
									Auf die Daten fr Minuten, Stunden und Datum folgt bei DCF77 ein Parittsbit.
									Um die Paritt zu prfen, werden die gesetzten Bits eines Datenpakets gezhlt.
									DCF77 verwendet gerade Paritt, d.h. ist die Anzahl der gesetzten Bits gerade,
									ist das gesendete Parittsbit 0. Diese Methode prft dies und gibt bei korrekter
									Paritt "true", sonst "false" zurck.
	Parameter:		Input:	parity	Parittsbit des zu prfenden Datenpakets. (0 oder 1)
							data	Das zu prfende Datenpaket.
					Return:	true	Parittsprfung OK
							false	Parittsprfung nicht OK 
 -------------------------------------------------------------------------------------------------------------------- */
 bool checkParity(uint8_t parity, uint32_t data)
 {
	int count;
	for (count = 0; data != 0; count++, data &= data-1);
	if(count & 1)		// uneven
	{
		count = 1;
	}
	else				// even
	{
		count = 0;
	}
	if (count == parity)
	{
		return true;
	}
	else 
	{
		return false;
	}	
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		checkParityTime:	Prft unter Verwendung der Methode checkParity ob die Paritt von Stunden und
										Minuten korrekt sind. Ist eine der beiden berprfungen fehlerhaft, wird
										"false" zurckgegeben und die Empfangenen Daten werden verworfen.
	Parameter:		Return:		true	Parittsprfung OK
								false	Parittsprfung nicht OK	 
 -------------------------------------------------------------------------------------------------------------------- */
 bool checkParityTime(void)
 {
	if (checkParity((dcf_receivedData_min>>DCF_MIN_PARI_MASK), (dcf_receivedData_min & DCF_MIN_MASK)) == true)
	{
		if (checkParity((dcf_receivedData_h>>DCF_H_PARI_MASK), (dcf_receivedData_h & DCF_H_MASK)) == true)
		{
			return true;
		}
		else
		{
			dcf_receivedData_min &= ~(0xFF & DCF_H_MASK);
			return false;
		}
	}
	else
	{
		dcf_receivedData_min &= ~(0xFF & DCF_MIN_MASK);
		return false;
	}
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		checkParityDate:	Prft unter Verwendung der Methode checkParity ob die Paritt des empfangenen 
										Datums korrekt ist. Ist die berprfung fehlerhaft, wird "false" zurckgegeben
										und die Empfangenen Daten werden verworfen.										
	Parameter:		Return:		true	Parittsprfung OK
								false	Parittsprfung nicht OK	 
 -------------------------------------------------------------------------------------------------------------------- */
 bool checkParityDate(void)
 {
	if (checkParity((dcf_receivedData_parity_date), ( (dcf_receivedData_date_day<<16) + (dcf_receivedData_weekday<<13) + (dcf_receivedData_date_month<<8) + (dcf_receivedData_date_year) )) == true)
	{
		return true;
	}
	else
	{
		dcf_receivedData_weekday = 0;
		dcf_receivedData_date_day = 0;
		dcf_receivedData_date_month = 0;
		dcf_receivedData_date_year = 0;
		return false;
	}
 }

 /* --------------------------------------------------------------------------------------------------------------------
	PIOA interrupt Handler		dcf_data_handler: Interpretation der Signale vom DCF-Modul.
								Zunchst wird geprft, ob das Signal an PA4 angekommen ist, ob es sich um ein
								fehlerfreies Signal handelt und ob der Empfang der Uhrzeit und des Datums bereit 
								abgeschlossen ist. Ist eines davon der Fall, wird alles Weitere bersprungen.
								Der Interrupt Handler synchronisiert darber hinaus den Empfang auf das erste Bit,
								prft empfangene Pakete auf Vollstndigkeit, ruft die Methode zur Parittsprfung auf,
								interpretiert und speichert die empfangenen Daten bitweise in einen Zwischenspeicher
								und gibt Auskunft ber den aktuellen Verlauf des DCF-Empfangs ber das LCD-Display.
 -------------------------------------------------------------------------------------------------------------------- */
 void dcf_data_handler(uint32_t id, uint32_t index)
 {
	if ((id == DCF_DATA_ID) && (index == DCF_DATA_MASK))	
	{
		if (dcf_invalidSignal == false && dcf_completed == false)
		{			
			if (pio_get(DCF_DATA_PIO, PIO_TYPE_PIO_INPUT, DCF_DATA_MASK))	// rising edge on DCF-DATA pin
			{		
				dcf_detectBitCounter = 0;			// counter for measuring the length of a single peek 
				
				if (dcf_syncCounter > 1500)			// sync only possible if no signal for more than 1,5s
				{
					if (dcf_synced == false)		// DCF not synced and sync recognized
					{
						dcf_synced = true;
						LCD_Clear();
						LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "DCF synced.");
						LCD_WriteLineToLCD(LINE2, TEXT_LEFT, "wait for data...");
						LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
						lcd_setCursor(0, LINE4);
						Write_char_lcd(LCD_MENU_CURSOR);
						if (uart_dcfActive == true)
						{
							uart_writeSynced();
						}
					}
					else							// DCF already synced and sync recognized
					{
						if (dcf_bitCounter != 59)	// not every single bit has been received -> need to get sync again
						{
							dcf_synced = false;		
							dcf_syncCounter = 1;
							dcf_invalidSync = true;
							LCD_Clear();
							LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "invalid data.");
							LCD_WriteLineToLCD(LINE2, TEXT_LEFT, "sync required.");
							LCD_WriteLineToLCD(LINE3, TEXT_LEFT, "wait for sync...");
							LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
							lcd_setCursor(0, LINE4);
							Write_char_lcd(LCD_MENU_CURSOR);
							if (uart_dcfActive == true)
							{
								uart_writeSyncRequ();
							}
						}
						else						// receive completely -> check parity
						{
							if (dcf_validTimeReceived == false && checkParityTime() == true)
							{
								updateTime();
								dcf_validTimeReceived = true;											
							}

							if (dcf_validDateReceived == false && checkParityDate() == true)
							{
								updateDate();
								dcf_validDateReceived = true;										
							}
							if (dcf_validTimeReceived == false || dcf_validDateReceived == false)
							{
								LCD_Clear();
								LCD_WriteLineToLCD(LINE3, TEXT_CENTER, "wait for data...");
								LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
								lcd_setCursor(0, LINE4);
								Write_char_lcd(LCD_MENU_CURSOR);
								if (dcf_validTimeReceived == false)
								{
									LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "time - invalid");
								}
								else
								{
									LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "time - OK");
								}
								if (dcf_validDateReceived == false)
								{
									LCD_WriteLineToLCD(LINE2, TEXT_CENTER, "date - invalid");
								}
								else
								{
									LCD_WriteLineToLCD(LINE2, TEXT_CENTER, "date - OK");
								}
								if (uart_dcfActive == true)
								{
									uart_writeDataInvalid();
								}
							}
						}
					}	
					dcf_bitCounter = 0;				
				}								
			} 
			else															// falling edge on DCF-DATA pin
			{	
				if (dcf_detectBitCounter > 100 && dcf_detectBitCounter < 134)		// received data = 0
				{
					gpio_toggle_pin(LED1);
					dcf_detectedBit = 0;
					if (dcf_signalTestDetection == true)					// signal test at the beginning is running -> count correkt signals
					{
						dcf_validSignalCounter++;
					}							
				}
				else if (dcf_detectBitCounter > 200 && dcf_detectBitCounter < 240)	// received data = 1
				{
					gpio_toggle_pin(LED2);	
					dcf_detectedBit = 1;
					if (dcf_signalTestDetection == true)					// signal test at the beginning is running -> count correkt signals
					{
						dcf_validSignalCounter++;
					}
				}

				dcf_errorDetection = dcf_detectBitCounter;
				dcf_syncCounter = 0;

				if (dcf_synced == true)										// if DCF is synced interpret and save the received Bit
				{
					getReceivedBit();
				}
			}
		}
	}
 }