/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	init.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

#include "compiler.h"
#include "asf.h"
#include "board.h"
#include "conf_board.h"
#include "gpio.h"
#include "ioport.h"
#include "lcd.h"
#include "dcf.h"
#include "rfm.h"

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		board_init: Initialisiert alle verwendeten Pins fr die beiden LEDs, das LCD-Display, den Summer
								den ENC-Controller, die SD-Karte und das DCF-Modul.
 -------------------------------------------------------------------------------------------------------------------- */

void board_init(void)
{
#ifndef CONF_BOARD_KEEP_WATCHDOG_AT_INIT
	/* Disable the watchdog */
	WDT->WDT_MR = WDT_MR_WDDIS;
#endif
	ioport_init();

	/* Configure LED pins */
	gpio_configure_pin(LED1_GPIO, LED1_FLAGS);
	gpio_configure_pin(LED2_GPIO, LED2_FLAGS);

	/* Configure LCD pins */
	ioport_enable_port(IOPORT_PIOC,LCD_MASK_ALL);
	ioport_set_port_dir(IOPORT_PIOC, LCD_MASK_ALL, IOPORT_DIR_OUTPUT);
	ioport_set_port_level(IOPORT_PIOC,LCD_MASK_ALL,0x00);

	/* Configure SUMMER pin */
	ioport_set_pin_dir(SUMMER, IOPORT_DIR_OUTPUT);
	ioport_set_pin_level(SUMMER, IOPORT_PIN_LEVEL_LOW);

	/* Configure SPI pins */
	gpio_configure_pin(SPI_MISO_GPIO, SPI_MISO_FLAGS);
	gpio_configure_pin(SPI_MOSI_GPIO, SPI_MOSI_FLAGS);
	gpio_configure_pin(SPI_SPCK_GPIO, SPI_SPCK_FLAGS);
	gpio_configure_pin(SPI_NPCS0_GPIO, SPI_NPCS0_FLAGS);
	gpio_configure_pin(SPI_NPCS1_GPIO, SPI_NPCS1_FLAGS);
	gpio_configure_pin(SPI_NPCS3_GPIO, SPI_NPCS3_FLAGS);

	/* Configure ENC INT-Pin & interrupt */
	ioport_set_pin_dir(ENC_INT, IOPORT_DIR_INPUT);
	ioport_set_pin_sense_mode(ENC_CLK, IOPORT_SENSE_FALLING);
	ioport_enable_pin(ENC_INT);

	/* Configure HSMCI pins */
	gpio_configure_pin(PIN_HSMCI_MCCDA_GPIO, PIN_HSMCI_MCCDA_FLAGS);
	gpio_configure_pin(PIN_HSMCI_MCCK_GPIO, PIN_HSMCI_MCCK_FLAGS);
	gpio_configure_pin(PIN_HSMCI_MCDA0_GPIO, PIN_HSMCI_MCDA0_FLAGS);
	gpio_configure_pin(PIN_HSMCI_MCDA1_GPIO, PIN_HSMCI_MCDA1_FLAGS);
	gpio_configure_pin(PIN_HSMCI_MCDA2_GPIO, PIN_HSMCI_MCDA2_FLAGS);
	gpio_configure_pin(PIN_HSMCI_MCDA3_GPIO, PIN_HSMCI_MCDA3_FLAGS);
	/* Configure SD/MMC card detect pin */
	gpio_configure_pin(SD_MMC_0_CD_GPIO, SD_MMC_0_CD_FLAGS);
	/* Configure SD/MMC sc pin */
	gpio_configure_pin(SD_MMC_GND_SC_GPIO, SD_MMC_GND_SC_FLAGS);

	/* Configure DCF pins */
	pio_set_input(DCF_DATA_PIO, DCF_DATA_MASK, PIO_DEFAULT);
	pio_handler_set(DCF_DATA_PIO, DCF_DATA_ID, DCF_DATA_MASK, PIO_IT_EDGE, dcf_data_handler);
	pio_enable_interrupt(DCF_DATA_PIO, DCF_DATA_MASK);
	gpio_configure_pin(DCF_PON_GPIO, DCF_PON_FLAGS);

	/* Configure RFM pins */
	pio_set_input(RFM_IRQ_PIO, RFM_IRQ_MASK, PIO_DEFAULT);
	pio_handler_set(RFM_IRQ_PIO, RFM_IRQ_ID, RFM_IRQ_MASK, PIO_IT_FALL_EDGE, rfm_handler);
	pio_enable_interrupt(RFM_IRQ_PIO, RFM_IRQ_MASK);
	gpio_configure_pin(RFM_CE_GPIO, RFM_CE_FLAGS);
}
