/*	
*	Pollin Electronic GmbH
*	Max-Pollin-Strae 1			
*	85104 Pfrring 
*	http://www.pollin.de
*
*	Project:	RAPITUS SAMxS Evaluationboard 
*  Filename:	menu.c 
*
*  Author:		Leonhard Hesse
*
*  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
*
*  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
*  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
*  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
*  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
*/

#include "asf.h"
#include "string.h"
#include "lcd.h"
#include "buttons_adc.h"
#include "uart.h"
#include "uart_menu.h"
#include "dcf.h"
#include "user.h"
#include "rfm.h"

#if defined RFM_ADDON && defined RFM_TRANSMITTER
	bool lcd_rfm_transmitter = true;
	bool lcd_rfm_receiver = false;
#elif defined RFM_ADDON && defined RFM_RECEIVER
	bool lcd_rfm_transmitter = false;
	bool lcd_rfm_receiver = true;
#else
	bool lcd_rfm_transmitter = false;
	bool lcd_rfm_receiver = false;
#endif
#ifdef DCF_ADDON
	bool lcd_dcf_used = true;
#else
	bool lcd_dcf_used = false;
#endif

uint8_t string[23] = "Hello! ";

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		manageMenu:	Verwaltung des Mens
					Prft anhand mehrerer Variablen die aktuelle Menstruktur und stellt anschlieend die 
					aktualisierten Daten fr das LC-Display zur Verfgung und lst eine Update-Methode aus.
					Prft ob eine Aktion notwendig ist und fhrt dies gegebenenfalls aus.

	Parameter:		Input:	int Button:	Wert der den bettigten Button darstellt.

	Variablen:		selMenu:		Reprsentiert das aktuell markierte Element im Hauptmen.
					selSubMenu:		Reprsentiert das aktuell markierte Element im Untermen 1.
					selSubSubMenu:	Reprsentiert das aktuell markierte Element im Untermen 2.
					cursorLine:		Reprsentiert die aktuelle Zeile, in der sich der Cursor befindet.
					scrollLine:		Reprsentiert die Anzahl der Zeilen, um die das Display nach oben verschoben ist.
 -------------------------------------------------------------------------------------------------------------------- */
 void manageMenu(int button)
 {
	 switch (button)
	 {
		 case BUTTON_UP:

		 /***** HAUPTMEN *****/

		 if (lcd_subMenu == false && lcd_subSubMenu == false && lcd_lastMenu == false)
		 {
			 if (lcd_cursorLine != LINE2)
			 {
				 lcd_cursorLine--;
				 lcd_selMenu--;
				 updateMenu();
			 }
			 else
			 {
				 lcd_scrollLine--;
				 lcd_selMenu--;
				 if (lcd_rfm_receiver == true || lcd_rfm_transmitter == true)
				 {
					switch (lcd_scrollLine)
					{
						case 0:
						lcd_selMenu = 1;
						lcd_scrollLine = 1;
						break;
						case 1:
						strncpy(menuLine1, MENU_ITEM_1_LED, sizeof(menuLine1));
						strncpy(menuLine2, MENU_ITEM_2_SUM, sizeof(menuLine2));
						strncpy(menuLine3, MENU_ITEM_3_RS232, sizeof(menuLine3));
						break;
						case 2:
						strncpy(menuLine1, MENU_ITEM_2_SUM, sizeof(menuLine1));
						strncpy(menuLine2, MENU_ITEM_3_RS232, sizeof(menuLine2));
						strncpy(menuLine3, MENU_ITEM_4_TIME, sizeof(menuLine3));
						break;
					}
				 }
				 else
				 {
					 switch (lcd_scrollLine)
					 {
						 case 0:
						 lcd_selMenu = 1;
						 lcd_scrollLine = 1;
						 break;
						 case 1:
						 strncpy(menuLine1, MENU_ITEM_1_LED, sizeof(menuLine1));
						 strncpy(menuLine2, MENU_ITEM_2_SUM, sizeof(menuLine2));
						 strncpy(menuLine3, MENU_ITEM_3_RS232, sizeof(menuLine3));
						 break;
					 }
				 }
				 updateMenu();
			 }
		 }

		 /***** UNTERMEN - 1 *****/

		 else if (lcd_subMenu == true && lcd_subSubMenu == false && lcd_lastMenu == false)
		 {
			 switch (lcd_selMenu)
			 {
				 case MENU_LED:
				 if (lcd_cursorLine != LINE3)
				 {
					 lcd_cursorLine--;
					 lcd_selSubMenu--;
					 updateSubMenu();
				 }
				 else
				 {
					 lcd_scrollLine--;
					 lcd_selSubMenu--;
					 switch (lcd_scrollLine)
					 {
						 case 0:
						 lcd_scrollLine = 1;
						 lcd_selSubMenu = 1;
						 break;
						 case 1:
						 strncpy(subMenuLine1, MENU_SUBITEM_LED_1, sizeof(subMenuLine1));
						 strncpy(subMenuLine2, MENU_SUBITEM_LED_2, sizeof(subMenuLine2));
						 break;
					 }
					 updateSubMenu();
				 }
				 break;
				 case MENU_SUMMER:
				 if (lcd_cursorLine != LINE3)
				 {
					 lcd_cursorLine--;
					 lcd_selSubMenu--;
					 updateSubMenu();
				 }
				 break;
				 case MENU_RS232:
				 if (lcd_cursorLine != LINE3)
				 {
					 lcd_cursorLine--;
					 lcd_selSubMenu--;
					 updateSubMenu();
				 }
				 else
				 {
					 lcd_scrollLine--;
					 lcd_selSubMenu--;
					 switch (lcd_scrollLine)
					 {
						 case 0:
						 lcd_scrollLine = 1;
						 lcd_selSubMenu = 1;
						 break;
						 case 1:
						 strncpy(subMenuLine1, MENU_SUBITEM_RS232_1, sizeof(subMenuLine1));
						 strncpy(subMenuLine2, MENU_SUBITEM_RS232_2, sizeof(subMenuLine2));
						 break;
					 }
					 updateSubMenu();
				 }
				 break;
				 case MENU_TIME:
				 if (lcd_cursorLine != LINE3)
				 {
					 lcd_cursorLine--;
					 lcd_selSubMenu--;
					 updateSubMenu();
				 }
				 else
				 {
					 lcd_scrollLine--;
					 lcd_selSubMenu--;			
					 if (lcd_dcf_used == true)
					 {
						 switch (lcd_scrollLine)
						 {
							 case 0:
							 lcd_scrollLine = 1;
							 lcd_selSubMenu = 1;
							 break;
							 case 1:
							 strncpy(subMenuLine1, MENU_SUBITEM_TIME_1, sizeof(subMenuLine1));
							 strncpy(subMenuLine2, MENU_SUBITEM_TIME_2, sizeof(subMenuLine2));
							 break;
							 case 2:
							 strncpy(subMenuLine1, MENU_SUBITEM_TIME_2, sizeof(subMenuLine1));
							 strncpy(subMenuLine2, MENU_SUBITEM_TIME_3, sizeof(subMenuLine2));
							 break;
						 }
					 }
					 else
					 {
						switch (lcd_scrollLine)
						{
							case 0:
							lcd_scrollLine = 1;
							lcd_selSubMenu = 1;
							break;
							case 1:
							strncpy(subMenuLine1, MENU_SUBITEM_TIME_2, sizeof(subMenuLine1));
							strncpy(subMenuLine2, MENU_SUBITEM_TIME_3, sizeof(subMenuLine2));
							break;
						 }
					 }
					 updateSubMenu();
				 }
				 break;

				 case MENU_RFM:
				 if (lcd_cursorLine != LINE3 && lcd_rfm_transmitter == true)
				 {
					 lcd_cursorLine--;
					 lcd_selSubMenu--;
					 updateSubMenu();
				 }
				 break;
			 }
		 }

		 /***** UNTERMEN - 2 *****/

		 else if (lcd_subMenu == true && lcd_subSubMenu == true && lcd_lastMenu == false)
		 {
			 switch (lcd_selMenu)
			 {
				 case MENU_RS232:
				 if (lcd_cursorLine != LINE3)
				 {
					 lcd_cursorLine--;
					 lcd_selSubSubMenu--;
					 updateSubMenu();
				 }
				 else
				 {
					 lcd_scrollLine--;
					 lcd_selSubSubMenu--;
					 switch (lcd_scrollLine)
					 {
						 case 0:
						 lcd_scrollLine = 1;
						 lcd_selSubSubMenu = 1;
						 break;
						 case 1:
						 strncpy(subMenuLine1, MENU_SUBSUBITEM_RS232_1, sizeof(subMenuLine1));
						 strncpy(subMenuLine2, MENU_SUBSUBITEM_RS232_2, sizeof(subMenuLine2));
						 break;
					 }
					 updateSubMenu();
				 }
				 break;
			 }
		 }
		 
		 break;

		 case BUTTON_LEFT:

		 /***** UNTERMEN - 2 *****/
		 if (lcd_subMenu == true && lcd_subSubMenu == true && lcd_lastMenu == false)
		 {
			 switch (lcd_selMenu)
			 {
				 case MENU_TIME:
				 switch (selTimeSetting)
				 {
					 case 0:		//Hour selected
					 if (manualTime_hour == 0){manualTime_hour = 24;}
					 manualTime_hour--;	 
					 updateManualTimeDateSetting(TIME);
					 break;
					 case 1:		//Minute selected
					 if (manualTime_min == 0){manualTime_min = 60;}
					 manualTime_min--;
					 updateManualTimeDateSetting(TIME);
					 break;
					 case 2:		//Day selected
					 manualDate_day--;
					 if (manualDate_day < 1){manualDate_day = 31;}
					 updateManualTimeDateSetting(DATE);
					 break;
					 case 3:		//Month selected
					 manualDate_month--;
					 if (manualDate_month < 1){manualDate_month = 12;}
					 updateManualTimeDateSetting(DATE);
					 break;
					 case 4:		//Year selected
					 manualDate_year--;
					 if (manualDate_year < 1901){manualDate_year = 2099;}
					 updateManualTimeDateSetting(DATE);
					 break;
					 case 5:
					 manualDate_week--;
					 if (manualDate_week < 1) {manualDate_week = 7;}
					 updateManualTimeDateSetting(WEEK);
					 break;
				 }
				 break;
			 }
		 }
		 break;

		 case BUTTON_RETURN:

		 /***** HAUPTMEN *****/

		 if (lcd_subMenu == false && lcd_subSubMenu == false && lcd_lastMenu == false)
		 {
			 lcd_subMenu = true;
			 lcd_selSubMenu = 1;
			 lcd_cursorLine = LINE3;
			 lcd_scrollLine = 1;

			 switch (lcd_selMenu)
			 {
				 case MENU_LED:
				 strncpy(selSubMenuText, MENU_ITEM_1_LED, sizeof(selSubMenuText));
				 strncpy(subMenuLine1, MENU_SUBITEM_LED_1, sizeof(subMenuLine1));
				 strncpy(subMenuLine2, MENU_SUBITEM_LED_2, sizeof(subMenuLine2));
				 if (uart_subMenu == false)
				 {
					uart_getMenuChoice('1');
				 }			 
				 break;
				 case MENU_SUMMER:
				 strncpy(selSubMenuText, MENU_ITEM_2_SUM, sizeof(selSubMenuText));
				 strncpy(subMenuLine1, MENU_SUBITEM_SUMMER, sizeof(subMenuLine1));
				 strncpy(subMenuLine2, MENU_SUBITEM_BACK, sizeof(subMenuLine2));
				 if (uart_subMenu == false)
				 {
					uart_getMenuChoice('2');
				 }
				 break;
				 case MENU_RS232:
				 strncpy(selSubMenuText, MENU_ITEM_3_RS232, sizeof(selSubMenuText));
				 strncpy(subMenuLine1, MENU_SUBITEM_RS232_1, sizeof(subMenuLine1));
				 strncpy(subMenuLine2, MENU_SUBITEM_RS232_2, sizeof(subMenuLine2));
				 break;
				 case MENU_TIME:
				 if (lcd_dcf_used)
				 {
					strncpy(selSubMenuText, MENU_ITEM_4_TIME, sizeof(selSubMenuText));
					strncpy(subMenuLine1, MENU_SUBITEM_TIME_1, sizeof(subMenuLine1));
					strncpy(subMenuLine2, MENU_SUBITEM_TIME_2, sizeof(subMenuLine2));
				 }
				 else
				 {
					strncpy(selSubMenuText, MENU_ITEM_4_TIME, sizeof(selSubMenuText));
					strncpy(subMenuLine1, MENU_SUBITEM_TIME_2, sizeof(subMenuLine1));
					strncpy(subMenuLine2, MENU_SUBITEM_TIME_3, sizeof(subMenuLine2));
				 }
				 if (uart_subMenu == false)
				 {
					uart_getMenuChoice('4');
				 }
				 break;
				 case MENU_RFM:
				 if (lcd_rfm_receiver == true)
				 {
					LCD_Clear();
					LCD_WriteLineToLCD(LINE1, TEXT_CENTER, MENU_SUBITEM_RFM_RX);
					LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
					lcd_setCursor(0, LINE4);
					Write_char_lcd(LCD_MENU_CURSOR);
					lcd_showReceivedDataRFM = true;
				 }
				 else if (lcd_rfm_transmitter == true)
				 {
					strncpy(selSubMenuText, MENU_ITEM_5_RFM, sizeof(selSubMenuText));
					strncpy(subMenuLine1, MENU_SUBITEM_RFM_TX, sizeof(subMenuLine1));
					strncpy(subMenuLine2, MENU_SUBITEM_BACK, sizeof(subMenuLine2));
					updateSubMenu();
				 }
				 if (uart_subMenu == false)
				 {
					 uart_getMenuChoice('5');
				 }
				 break;
			 }
			 if (lcd_selMenu != MENU_RFM)
			 {
				updateSubMenu();
			 }
		 }

		 /***** UNTERMEN - 1 *****/

		 else if (lcd_subMenu == true && lcd_subSubMenu == false && lcd_lastMenu == false)
		 {
			 switch (lcd_selMenu)
			 {
				 case MENU_LED:
				 switch (lcd_selSubMenu)
				 {
					 case 1:
					 gpio_toggle_pin(LED1);
					 break;
					 case 2:
					 gpio_toggle_pin(LED2);
					 break;
					 case 3:
					 gpio_toggle_pin(LED1);
					 gpio_toggle_pin(LED2);
					 break;
					 case 4:
					 show_LCD_MainMenu();
					 break;
				 }
				 break;
				 case MENU_SUMMER:
				 switch (lcd_selSubMenu)
				 {
					 case 1:
					 if (summerStart == true)
					 {
						 summerStart = false;
					 }
					 else
					 {
						 summerStart = true;
					 }
					 break;
					 case 2:
					 show_LCD_MainMenu();
					 break;
				 }
				 break;
				 case MENU_RS232:
				 lcd_subSubMenu = true;
				 lcd_selSubSubMenu = 1;
				 lcd_cursorLine = LINE3;
				 lcd_scrollLine = 1;
				 switch (lcd_selSubMenu) //abhngig von der Auswahl im Sub-Men -> SubSub-Men anzeigen
				 {
					 case 1:
					 strncpy(selSubMenuText, MENU_SUBITEM_RS232_1, sizeof(selSubMenuText));
					 strncpy(subMenuLine1, MENU_SUBSUBITEM_RS232_1, sizeof(subMenuLine1));
					 strncpy(subMenuLine2, MENU_SUBSUBITEM_RS232_2, sizeof(subMenuLine2));
					 updateSubMenu();
					 break;
					 case 2:
					 strncpy(selSubMenuText, MENU_SUBITEM_RS232_2, sizeof(selSubMenuText));
					 strncpy(subMenuLine1, MENU_SUBSUBITEM_RS232_1, sizeof(subMenuLine1));
					 strncpy(subMenuLine2, MENU_SUBSUBITEM_RS232_2, sizeof(subMenuLine2));
					 updateSubMenu();
					 break;
					 case 3:
					 show_LCD_MainMenu();
					 break;
				 }
				 break;

				 case MENU_TIME:
				 lcd_subSubMenu = true;
				 if (lcd_dcf_used)
				 {
					switch (lcd_selSubMenu)
					{
						case 1:
						dcf_init();
						lcd_selSubSubMenu = 1;
						break;
						case 2:
						lcd_selSubSubMenu = 2;
						selTimeSetting = 0;
						uint32_t dummySec;
						rtc_get_time(RTC, &manualTime_hour, &manualTime_min, &dummySec);
						rtc_get_date(RTC, &manualDate_year, &manualDate_month, &manualDate_day, &manualDate_week);
						LCD_Clear();
						updateManualTimeDateSetting(TIME);
						lcd_setCursor(7, LINE2);
						Write_char_lcd(LCD_TIME_CURSOR_R);
						lcd_setCursor(7, LINE4);
						Write_char_lcd(LCD_TIME_CURSOR_L);
						break;
						case 3:
						lcd_selSubSubMenu = 3;
						lcd_showTime = true;
						break;
						case 4:
						show_LCD_MainMenu();
						break;
					}
				 break;
				 }
				 else
				 {
					switch (lcd_selSubMenu)
					{
						case 1:
						lcd_selSubSubMenu = 2;
						selTimeSetting = 0;
						uint32_t dummySec;
						rtc_get_time(RTC, &manualTime_hour, &manualTime_min, &dummySec);
						rtc_get_date(RTC, &manualDate_year, &manualDate_month, &manualDate_day, &manualDate_week);
						LCD_Clear();
						updateManualTimeDateSetting(TIME);
						lcd_setCursor(7, LINE2);
						Write_char_lcd(LCD_TIME_CURSOR_R);
						lcd_setCursor(7, LINE4);
						Write_char_lcd(LCD_TIME_CURSOR_L);
						break;
						case 2:
						lcd_selSubSubMenu = 3;
						lcd_showTime = true;
						break;
						case 3:
						show_LCD_MainMenu();
						break;
					}
				 break;

				 case MENU_RFM:
				 if (lcd_rfm_receiver == true)
				 {
					 lcd_showReceivedDataRFM = false;
					 show_LCD_MainMenu();
				 }
				 else if (lcd_rfm_transmitter == true)
				 {
					switch (lcd_selSubMenu)
					{
						case 1:
						rfm_sendArray(string, MODULE_PAYLOAD);
						break;

						case 2:
						show_LCD_MainMenu();
						break;
					}
				 }
				 break;
				 }
				 
			 }
		 }

		 /***** UNTERMEN - 2 *****/

		 else if (lcd_subMenu == true && lcd_subSubMenu == true && lcd_lastMenu == false)		//aktuell wird im Men ein SubSub-Men angezeigt
		 {
			 switch (lcd_selMenu)
			 {
				 case MENU_RS232:
				 switch (lcd_selSubMenu)
				 {
					 case 1:									//RS232 J2
					 switch (lcd_selSubSubMenu)
					 {
						 case 1:								//Transmit Data
						 uart_write_string(UART0, "Hello! My name is Sam. ");
						 break;
						 case 2:								//Receive Data
						 LCD_Clear();
						 LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "Received data J2");
						 LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
						 lcd_setCursor(0, LINE4);
						 Write_char_lcd(LCD_MENU_CURSOR);
						 lcd_lastMenu = true;
						 lcd_showReceivedData = true;
						 break;
						 case 3:								// Exit
						 show_LCD_MainMenu();
						 break;
					 }
					 break;
					 case 2:									//RS232 J3
					 switch (lcd_selSubSubMenu)
					 {
						 case 1:								//Transmit Data
						 uart_write_string(UART1, "Hello! My name is Sam. ");
						 break;
						 case 2:								//Receive Data
						 LCD_Clear();
						 LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "Received data J3");
						 LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
						 lcd_setCursor(0, LINE4);
						 Write_char_lcd(LCD_MENU_CURSOR);
						 lcd_lastMenu = true;
						 lcd_showReceivedData = true;
						 break;
						 case 3:								// Exit
						 show_LCD_MainMenu();
						 break;
					 }					
				 }
				 break;
				 case MENU_TIME:		
				 switch (lcd_selSubSubMenu)
				 {
					case 1:	//back; display messages and menu managed by dcf.c & RTC_Handler
					dcf_stop();
					lcd_showTime = false;
					show_LCD_MainMenu();
					break;
					case 2:	//is in manual time setting -> switch to next setting
					selTimeSetting++;
					switch (selTimeSetting)
					{
						case 1:
						lcdClearLine(LINE2);
						lcdClearLine(LINE4);
						lcd_setCursor(10, LINE2);
						Write_char_lcd(LCD_TIME_CURSOR_R);
						lcd_setCursor(10, LINE4);
						Write_char_lcd(LCD_TIME_CURSOR_L);
						break;
						case 2:
						lcdClearLine(LINE2);
						lcdClearLine(LINE4);
						updateManualTimeDateSetting(DATE);
						lcd_setCursor(7, LINE2);
						Write_char_lcd(LCD_TIME_CURSOR_R);
						lcd_setCursor(7, LINE4);
						Write_char_lcd(LCD_TIME_CURSOR_L);
						break;
						case 3:
						lcdClearLine(LINE2);
						lcdClearLine(LINE4);
						lcd_setCursor(10, LINE2);
						Write_char_lcd(LCD_TIME_CURSOR_R);
						lcd_setCursor(10, LINE4);
						Write_char_lcd(LCD_TIME_CURSOR_L);
						break;
						case 4:
						lcdClearLine(LINE2);
						lcdClearLine(LINE4);
						lcd_setCursor(15, LINE2);
						Write_char_lcd(LCD_TIME_CURSOR_R);
						lcd_setCursor(15, LINE4);
						Write_char_lcd(LCD_TIME_CURSOR_L);
						break;
						case 5:
						lcdClearLine(LINE2);
						lcdClearLine(LINE4);
						updateManualTimeDateSetting(WEEK);
						lcd_setCursor(10, LINE2);
						Write_char_lcd(LCD_TIME_CURSOR_R);
						lcd_setCursor(10, LINE4);
						Write_char_lcd(LCD_TIME_CURSOR_L);
						break;
						case 6:
						LCD_Clear();
						LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "I store data..");
						LCD_WriteLineToLCD(LINE2, TEXT_LEFT, "please wait...");
						timeSetManualDone = true;
						rtc_set_time(RTC, manualTime_hour, manualTime_min, 0);
						rtc_set_date(RTC, manualDate_year, manualDate_month, manualDate_day, manualDate_week);
						break;
						case 7:
						timeSetManualDone = false;
						show_LCD_MainMenu();
						break;
					}
					break;
					case 3:
					lcd_showTime = false;
					show_LCD_MainMenu();
					break;
				}
			}
		 }

		 /***** UNTERMEN - 3 *****/

		 else if (lcd_lastMenu == true)
		 {
			 lcd_lastMenu = false;
			 lcd_showReceivedData = false;
			 show_LCD_MainMenu();
		 }
		 break;

		 case BUTTON_RIGHT:

		 /***** UNTERMEN - 2 *****/
		 if (lcd_subMenu == true && lcd_subSubMenu == true && lcd_lastMenu == false)
		 {
			switch (lcd_selMenu)
			{
			case MENU_TIME:
				switch (selTimeSetting)
				{
					case 0:		//Hour selected
					manualTime_hour++;
					if (manualTime_hour > 23) {manualTime_hour = 0;}
					updateManualTimeDateSetting(TIME);
					break;
					case 1:		//Minute selected
					manualTime_min++;
					if (manualTime_min > 59) {manualTime_min = 0;}
					updateManualTimeDateSetting(TIME);
					break;
					case 2:		//Day selected
					manualDate_day++;
					if (manualDate_day > 31) {manualDate_day = 1;}
					updateManualTimeDateSetting(DATE);
					break;
					case 3:		//Month selected
					manualDate_month++;
					if (manualDate_month > 12) {manualDate_month = 1;}
					updateManualTimeDateSetting(DATE);
					break;
					case 4:		//Year selected
					manualDate_year++;
					if (manualDate_year > 2099) {manualDate_year = 1901;}
					updateManualTimeDateSetting(DATE);
					break;
					case 5:		//Weekday select
					manualDate_week++;
					if (manualDate_week > 7) {manualDate_week = 1;}
					updateManualTimeDateSetting(WEEK);
					break;
				}
			break;
			}
		 }
		 break;
		 
		 case BUTTON_DOWN:

		 /***** HAUPTMEN *****/

		 if (lcd_subMenu == false && lcd_subSubMenu == false && lcd_lastMenu == false)			//aktuell ist Hauptmen
		 {
			 if (lcd_cursorLine != LINE4)
			 {
				 lcd_cursorLine++;
				 lcd_selMenu++;
				 updateMenu();
			 }
			 else
			 {
				 lcd_scrollLine++;
				 lcd_selMenu++;
				 if (lcd_rfm_receiver == true || lcd_rfm_transmitter == true)
				 {
					switch (lcd_scrollLine)
					{
						case 2:
						strncpy(menuLine1, MENU_ITEM_2_SUM, sizeof(menuLine1));
						strncpy(menuLine2, MENU_ITEM_3_RS232, sizeof(menuLine2));
						strncpy(menuLine3, MENU_ITEM_4_TIME, sizeof(menuLine3));
						break;
						case 3:
						strncpy(menuLine1, MENU_ITEM_3_RS232, sizeof(menuLine1));
						strncpy(menuLine2, MENU_ITEM_4_TIME, sizeof(menuLine2));
						strncpy(menuLine3, MENU_ITEM_5_RFM, sizeof(menuLine3));
						break;
						case 4:
						lcd_scrollLine = 3;
						lcd_selMenu = 5;
						break;
					}
				 }
				 else
				 {
					 switch (lcd_scrollLine)
					 {
						 case 2:
						 strncpy(menuLine1, MENU_ITEM_2_SUM, sizeof(menuLine1));
						 strncpy(menuLine2, MENU_ITEM_3_RS232, sizeof(menuLine2));
						 strncpy(menuLine3, MENU_ITEM_4_TIME, sizeof(menuLine3));
						 break;
						 case 3:
						 lcd_scrollLine = 2;
						 lcd_selMenu = 4;
						 break;
					 }
				 }
				 updateMenu();
			 }
		 }

		 /***** UNTERMEN - 1 *****/

		 else if (lcd_subMenu == true && lcd_subSubMenu == false && lcd_lastMenu == false)		//aktuell ist Sub-Men
		 {
			 switch (lcd_selMenu)
			 {
				 case MENU_LED:
					if (lcd_cursorLine != LINE4)
					{
						lcd_cursorLine++;
						lcd_selSubMenu++;
						updateSubMenu();
					}
					else
					{
						lcd_scrollLine++;
						lcd_selSubMenu++;
						switch (lcd_scrollLine)
						{
							case 2:
								strncpy(subMenuLine1, MENU_SUBITEM_LED_2, sizeof(subMenuLine1));
								strncpy(subMenuLine2, MENU_SUBITEM_LED_3, sizeof(subMenuLine2));
							break;

							case 3:
								strncpy(subMenuLine1, MENU_SUBITEM_LED_3, sizeof(subMenuLine1));
								strncpy(subMenuLine2, MENU_SUBITEM_BACK, sizeof(subMenuLine2));
							break;

							case 4:
								lcd_scrollLine = 3;
								lcd_selSubMenu = 4;
							break;
						}
						updateSubMenu();
					}
				 break;

				 case MENU_SUMMER:
					if (lcd_cursorLine != LINE4)
					{
						lcd_cursorLine++;
						lcd_selSubMenu++;
						updateSubMenu();
					}
				 break;

				 case MENU_RS232:
					if (lcd_cursorLine != LINE4)
					{
						lcd_cursorLine++;
						lcd_selSubMenu++;
						updateSubMenu();
					}
					else
					{
						lcd_scrollLine++;
						lcd_selSubMenu++;
						switch (lcd_scrollLine)
						{
							case 2:
								strncpy(subMenuLine1, MENU_SUBITEM_RS232_2, sizeof(subMenuLine1));
								strncpy(subMenuLine2, MENU_SUBITEM_BACK, sizeof(subMenuLine2));
							break;

							case 3:
								lcd_scrollLine = 2;
								lcd_selSubMenu = 3;
							break;
						}
						updateSubMenu();
					}
				 break;

				 case MENU_TIME:
				 if (lcd_cursorLine != LINE4)
				 {
					 lcd_cursorLine++;
					 lcd_selSubMenu++;
					 updateSubMenu();
				 }
				 else
				 {
					 lcd_scrollLine++;
					 lcd_selSubMenu++;
					 if (lcd_dcf_used == true)
					 {
						 switch (lcd_scrollLine)
						 {
							case 2:
							strncpy(subMenuLine1, MENU_SUBITEM_TIME_2, sizeof(subMenuLine1));
							strncpy(subMenuLine2, MENU_SUBITEM_TIME_3, sizeof(subMenuLine2));
							break;

							case 3:
							strncpy(subMenuLine1, MENU_SUBITEM_TIME_3, sizeof(subMenuLine1));
							strncpy(subMenuLine2, MENU_SUBITEM_BACK, sizeof(subMenuLine2));
							break;

							case 4:
							lcd_scrollLine = 3;
							lcd_selSubMenu = 4;
							break;
						 }
					 }
					 else
					 {
						 switch (lcd_scrollLine)
						 {
							case 2:
							strncpy(subMenuLine1, MENU_SUBITEM_TIME_3, sizeof(subMenuLine1));
							strncpy(subMenuLine2, MENU_SUBITEM_BACK, sizeof(subMenuLine2));
							break;

							case 3:
							lcd_scrollLine = 2;
							lcd_selSubMenu = 3;
							break;
						 }					 
					 }
					 updateSubMenu();
				 }
				 break;

				 case MENU_RFM:
				 if (lcd_cursorLine != LINE4 && lcd_rfm_transmitter == true)
				 {
					 lcd_cursorLine++;
					 lcd_selSubMenu++;
					 updateSubMenu();
				 }
			 }
		 }

		 /***** UNTERMEN - 2 *****/

		 else if (lcd_subMenu == true && lcd_subSubMenu == true && lcd_lastMenu == false)
		 {
			 switch (lcd_selMenu)
			 {
				 case MENU_RS232:
				 if (lcd_cursorLine != LINE4)
				 {
					 lcd_cursorLine++;
					 lcd_selSubSubMenu++;
					 updateSubMenu();
				 }
				 else
				 {
					 lcd_scrollLine++;
					 lcd_selSubSubMenu++;
					 switch (lcd_scrollLine)
					 {
						 case 2:
							strncpy(subMenuLine1, MENU_SUBSUBITEM_RS232_2, sizeof(subMenuLine1));
							strncpy(subMenuLine2, MENU_SUBITEM_BACK, sizeof(subMenuLine2));
						 break;
						 case 3:
							lcd_scrollLine = 2;
							lcd_selSubSubMenu = 3;
						 break;
					 }
					 updateSubMenu();
				 }
				 break;
			 }
		 }
		 break;
	 }
 }

 void show_LCD_MainMenu (void)
 {
	lcd_showTime = false;
	lcd_subMenu = false;
	lcd_subSubMenu = false;
	lcd_selMenu = 1;
	lcd_cursorLine = LINE2;
	lcd_scrollLine = 1;
	strncpy(menuLine1, MENU_ITEM_1_LED, sizeof(menuLine1));
	strncpy(menuLine2, MENU_ITEM_2_SUM, sizeof(menuLine2));
	strncpy(menuLine3, MENU_ITEM_3_RS232, sizeof(menuLine3));
	updateMenu();
	if (uart_selMenu != 0)
	{
		show_UART_MainMenu();
	}
 }