/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	mySpi.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

 #include "asf.h"
 #include "mySpi.h"

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		mySPI_rw:	Lesen und Schreiben von Daten am SPI-BUS
								Diese Methode sendet die bergebenen Daten an den ausgewhlten Slave oder empfngt Daten
								von diesem und gibt sie zurck.
	Parameter:		Input:		p_spi		SPI Base Address 
								us_data		Die zu sendenden Daten
								uc_pcs		Der ausgewhlte Empfnger (Chip select)
								us_last		Merker, ob weitere Daten folgen oder nicht (1 = letztes Paket)
					Return:		Wert des SPI-Empfangsregister
 -------------------------------------------------------------------------------------------------------------------- */
uint8_t mySpi_rw(Spi *p_spi, uint8_t us_data, uint8_t uc_pcs, uint8_t uc_last)
{
	uint32_t value;

	while (!(p_spi->SPI_SR & SPI_SR_TDRE)); 
	value = SPI_TDR_TD(us_data) | SPI_TDR_PCS(uc_pcs);
	if (uc_last) 
	{
		value |= SPI_TDR_LASTXFER;
	}
	p_spi->SPI_TDR = value;

	uint32_t reg_value;

	while (!(p_spi->SPI_SR & SPI_SR_RDRF)); 
	reg_value = p_spi->SPI_RDR;
	return (uint8_t) (reg_value & SPI_RDR_RD_Msk);
}

/* --------------------------------------------------------------------------------------------------------------------
	Methode:		mySPI_init:	Initialisierung des SPI-Bus
								Aktivierung und Initialisierung des SPI-Bus und der vorhandenen Teilnehmer (CS).
 -------------------------------------------------------------------------------------------------------------------- */
void mySpi_init(void)
{
	spi_enable_clock(SPI);
	spi_disable(SPI);
	spi_reset(SPI);
	spi_set_master_mode(SPI);
	spi_disable_mode_fault_detect(SPI);
	spi_set_variable_peripheral_select(SPI);

	spi_configure_cs_behavior(SPI, SPI_CS_RFM, SPI_CS_RISE_NO_TX);
	spi_set_clock_polarity(SPI, SPI_CS_RFM, SPI_CLK_POLARITY);
	spi_set_clock_phase(SPI, SPI_CS_RFM, SPI_CLK_PHASE);
	spi_set_bits_per_transfer(SPI, SPI_CS_RFM, SPI_CSR_BITS_8_BIT);
	spi_set_baudrate_div(SPI, SPI_CS_RFM,(sysclk_get_cpu_hz() / gs_ul_spi_clock));
	spi_set_transfer_delay(SPI, SPI_CS_RFM, SPI_DLYBS,SPI_DLYBCT);

	spi_configure_cs_behavior(SPI, SPI_CS_EXT, SPI_CS_RISE_NO_TX);
	spi_set_clock_polarity(SPI, SPI_CS_EXT, SPI_CLK_POLARITY);
	spi_set_clock_phase(SPI, SPI_CS_EXT, SPI_CLK_PHASE);
	spi_set_bits_per_transfer(SPI, SPI_CS_EXT, SPI_CSR_BITS_8_BIT);
	spi_set_baudrate_div(SPI, SPI_CS_EXT,(sysclk_get_cpu_hz() / gs_ul_spi_clock));
	spi_set_transfer_delay(SPI, SPI_CS_EXT, SPI_DLYBS,SPI_DLYBCT);

	spi_configure_cs_behavior(SPI, SPI_CS_ENC, SPI_CS_RISE_NO_TX);
	spi_set_clock_polarity(SPI, SPI_CS_ENC, SPI_CLK_POLARITY);
	spi_set_clock_phase(SPI, SPI_CS_ENC, SPI_CLK_PHASE);
	spi_set_bits_per_transfer(SPI, SPI_CS_ENC, SPI_CSR_BITS_8_BIT);
	spi_set_baudrate_div(SPI, SPI_CS_ENC,(sysclk_get_cpu_hz() / gs_ul_spi_clock));
	spi_set_transfer_delay(SPI, SPI_CS_ENC, SPI_DLYBS,SPI_DLYBCT);
	
	spi_enable(SPI);
}