/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	sam4s_evalBoard.h
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */


#ifndef SAM4S_EVALBOARD_H_
#define SAM4S_EVALBOARD_H_

#include "compiler.h"
#include "system_sam4s.h"

/** Board oscillator settings */
#define BOARD_FREQ_SLCK_XTAL			(32768U)
#define BOARD_FREQ_SLCK_BYPASS			(32768U)
#define BOARD_FREQ_MAINCK_XTAL			(12000000U)
#define BOARD_FREQ_MAINCK_BYPASS		(12000000U)

/** Master clock frequency */
#define BOARD_MCK						CHIP_FREQ_CPU_MAX

/** board main clock xtal startup time */
#define BOARD_OSC_STARTUP_US			15625

/** Name of the board */
#define BOARD_NAME "ARM-Evaluationboard"
/** Family definition */
#define sam4s
/** Core definition */
#define cortexm4

/*----------------------------------------------------------------------------*/
/** Pins ADC */
#define PIN_ADC0_AD8					{PIO_PA21X1_AD8, PIOA, ID_PIOA, PIO_INPUT, PIO_DEFAULT}
#define PINS_ADC PIN_ADC0_AD8

/** UART0 pins (UTXD0 and URXD0) definitions, PA9,PA10. */
#define PINS_UART0						(PIO_PA9A_URXD0 | PIO_PA10A_UTXD0)
#define PINS_UART0_FLAGS				(PIO_PERIPH_A	| PIO_DEFAULT)
#define PINS_UART0_MASK					(PIO_PA9A_URXD0 | PIO_PA10A_UTXD0)
#define PINS_UART0_PIO					PIOA
#define PINS_UART0_ID					ID_PIOA
#define PINS_UART0_TYPE					PIO_PERIPH_A
#define PINS_UART0_ATTR					PIO_DEFAULT

/** UART1 pins (UTXD1 and URXD1) definitions, PB2,PB3. */
#define PINS_UART1						(PIO_PB2A_URXD1 | PIO_PB3A_UTXD1)
#define PINS_UART1_FLAGS				(PIO_PERIPH_A	| PIO_DEFAULT)
#define PINS_UART1_MASK					(PIO_PB2A_URXD1 | PIO_PB3A_UTXD1)
#define PINS_UART1_PIO					PIOB
#define PINS_UART1_ID					ID_PIOB
#define PINS_UART1_TYPE					PIO_PERIPH_A
#define PINS_UART1_ATTR					PIO_DEFAULT

/** UART2 pins (TXD0 and RXD0) definitions, PA5,PA6. */
#define PINS_UART2						(PIO_PA5A_RXD0	| PIO_PA6A_TXD0)
#define PINS_UART2_FLAGS				(PIO_PERIPH_A	| PIO_DEFAULT)
#define PINS_UART2_MASK					(PIO_PA5A_RXD0 | PIO_PA6A_TXD0)
#define PINS_UART2_PIO					PIOA
#define PINS_UART2_ID					ID_PIOA
#define PINS_UART2_TYPE					PIO_PERIPH_A
#define PINS_UART2_ATTR					PIO_DEFAULT

/** LED #1 pin definition (LED1). */
#define LED_1_NAME						"LED1"
#define PIN_LED_1						{PIO_PC0, PIOC, ID_PIOC, PIO_OUTPUT_0, PIO_DEFAULT}
#define PIN_LED_1_MASK					PIO_PC0
#define PIN_LED_1_PIO					PIOC
#define PIN_LED_1_ID					ID_PIOC
#define PIN_LED_1_TYPE					PIO_OUTPUT_0
#define PIN_LED_1_ATTR					PIO_DEFAULT

#define LED1_GPIO						(PIO_PC0_IDX)
#define LED1_FLAGS						(PIO_OUTPUT_0 | PIO_DEFAULT)
#define LED1_ACTIVE_LEVEL				0
#define LED1							LED1_GPIO

/** LED #2 pin definition (LED2). */
#define LED_2_NAME						"LED2"
#define PIN_LED_2						{PIO_PC15, PIOC, ID_PIOC, PIO_OUTPUT_0, PIO_DEFAULT}
#define PIN_LED_2_MASK					PIO_PC15
#define PIN_LED_2_PIO					PIOC
#define PIN_LED_2_ID					ID_PIOC
#define PIN_LED_2_TYPE					PIO_OUTPUT_0
#define PIN_LED_2_ATTR					PIO_DEFAULT

#define LED2_GPIO						(PIO_PC15_IDX)
#define LED2_FLAGS						(PIO_OUTPUT_0 | PIO_DEFAULT)
#define LED2_ACTIVE_LEVEL				0
#define LED2							LED2_GPIO

#define BOARD_NUM_OF_LED 2

/** DCF receiver pin definition*/
#define DCF_DATA						{PIO_PA4, PIOA, ID_PIOA, PIO_INPUT, PIO_DEFAULT}
#define DCF_DATA_MASK					PIO_PA4
#define DCF_DATA_PIO					PIOA
#define DCF_DATA_ID						ID_PIOA
#define DCF_DATA_GPIO					(PIO_PA4_IDX)
#define DCF_DATA_DETECT_VALUE			1
#define DCF_PON							{PIO_PC18, ID_PIOC, PIO_OUTPUT_1, PIO_DEFAULT}
#define DCF_PON_MASK					PIO_PC18
#define DCF_PON_GPIO					(PIO_PC18_IDX)
#define DCF_PON_FLAGS					(PIO_OUTPUT_1 | PIO_DEFAULT)

/** SUMMER pin definition. */
#define SUMMER							IOPORT_CREATE_PIN	(PIOC, 19)

/** ENC controller pin definition */
#define ENC_CLK							IOPORT_CREATE_PIN	(PIOA, 22)
#define ENC_INT							IOPORT_CREATE_PIN	(PIOA, 0)
#define ENC_WOL							IOPORT_CREATE_PIN	(PIOC, 14)
#define ENC_CS							IOPORT_CREATE_PIN	(PIOA, 3)
#define SPI_NPCS3_GPIO					(PIO_PA3_IDX)
#define SPI_NPCS3_FLAGS					(PIO_PERIPH_B | PIO_DEFAULT)

/** SPI MISO pin definition. */
#define SPI_MISO_GPIO					(PIO_PA12_IDX)
#define SPI_MISO_FLAGS					(PIO_PERIPH_A | PIO_DEFAULT)
/** SPI MOSI pin definition. */
#define SPI_MOSI_GPIO					(PIO_PA13_IDX)
#define SPI_MOSI_FLAGS					(PIO_PERIPH_A | PIO_DEFAULT)
/** SPI SPCK pin definition. */
#define SPI_SPCK_GPIO					(PIO_PA14_IDX)
#define SPI_SPCK_FLAGS					(PIO_PERIPH_A | PIO_DEFAULT)
/** SPI chip select 0 pin definition. (Only one configuration is possible) */
#define SPI_NPCS0_GPIO					(PIO_PA11_IDX)
#define SPI_NPCS0_FLAGS					(PIO_PERIPH_A | PIO_DEFAULT)
/** SPI chip select 1 pin definition. (multiple configurations are possible) */
#define SPI_NPCS1_GPIO					(PIO_PC4_IDX)
#define SPI_NPCS1_FLAGS					(PIO_PERIPH_B | PIO_DEFAULT)
/** SPI chip select 2 pin definition. (multiple configurations are possible) */
#define SPI_NPCS2_GPIO					(PIO_PA30_IDX)
#define SPI_NPCS2_FLAGS					(PIO_PERIPH_A | PIO_DEFAULT)

/** HSMCI pins definition. */
/*! Number of slot connected on HSMCI interface */
#define CONF_BOARD_SD_MMC_HSMCI
#define SD_MMC_HSMCI_MEM_CNT			1
#define SD_MMC_HSMCI_SLOT_0_SIZE		4
#define PINS_HSMCI						{0x3fUL << 26, PIOA, ID_PIOA, PIO_PERIPH_C, PIO_PULLUP}
/** HSMCI MCCDA pin definition. */
#define PIN_HSMCI_MCCDA_GPIO            (PIO_PA28_IDX)
#define PIN_HSMCI_MCCDA_FLAGS           (PIO_PERIPH_C | PIO_DEFAULT)
/** HSMCI MCCK pin definition. */
#define PIN_HSMCI_MCCK_GPIO             (PIO_PA29_IDX)
#define PIN_HSMCI_MCCK_FLAGS            (PIO_PERIPH_C | PIO_DEFAULT)
/** HSMCI MCDA0 pin definition. */
#define PIN_HSMCI_MCDA0_GPIO            (PIO_PA30_IDX)
#define PIN_HSMCI_MCDA0_FLAGS           (PIO_PERIPH_C | PIO_DEFAULT)
/** HSMCI MCDA1 pin definition. */
#define PIN_HSMCI_MCDA1_GPIO            (PIO_PA31_IDX)
#define PIN_HSMCI_MCDA1_FLAGS           (PIO_PERIPH_C | PIO_DEFAULT)
/** HSMCI MCDA2 pin definition. */
#define PIN_HSMCI_MCDA2_GPIO            (PIO_PA26_IDX)
#define PIN_HSMCI_MCDA2_FLAGS           (PIO_PERIPH_C | PIO_DEFAULT)
/** HSMCI MCDA3 pin definition. */
#define PIN_HSMCI_MCDA3_GPIO            (PIO_PA27_IDX)
#define PIN_HSMCI_MCDA3_FLAGS           (PIO_PERIPH_C | PIO_DEFAULT)
/** SD/MMC card detect pin definition. */
#define PIN_HSMCI_CD					{PIO_PA25, PIOA, ID_PIOA, PIO_INPUT, PIO_PULLUP}
#define SD_MMC_0_CD_GPIO				(PIO_PA25_IDX)
#define SD_MMC_0_CD_PIO_ID				ID_PIOA
#define SD_MMC_0_CD_FLAGS				(PIO_INPUT | PIO_PULLUP)
#define SD_MMC_0_CD_DETECT_VALUE		0
/** SD/MMC sc pin definition. */
#define SD_MMC_GND_SC_GPIO				(PIO_PA23_IDX)
#define SD_MMC_GND_SC_FLAGS				(PIO_OUTPUT_0 | PIO_DEFAULT)

#endif /* SAM4S_EVALBOARD_H_ */