/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	uart.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

 #include "user.h"
 #include "asf.h"
 #include "uart.h"
 #include "lcd.h"
 #include "uart_menu.h"
 #include "rfm.h"

 #if defined RFM_RS232_J3 && defined RFM_TRANSMITTER
	bool uart_rfm_J3_def = true;
 #else
	bool uart_rfm_J3_def = false;
 #endif

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		myUart0_init:	Initialisiert UART0.
									Konfiguration der verwendeten Pins, Tack im PMC (Power Management) aktivieren,
									Baudrate und Modus einstellen eines Interrupt wenn daten empfangen werden.
 -------------------------------------------------------------------------------------------------------------------- */
 void myUart0_init (void)
 {
	pio_configure(PINS_UART0_PIO, PINS_UART0_TYPE, PINS_UART0_MASK, PINS_UART0_ATTR);

	pmc_enable_periph_clk(ID_UART0);

	const sam_uart_opt_t uart0_settings =
	{ sysclk_get_cpu_hz(), UART_SERIAL_BAUDRATE, UART_SERIAL_MODE };
	
	uart_init(UART0, &uart0_settings);					//Init UART1 and enable Rx and Tx
	
	uart_enable_interrupt(UART0, UART_IER_RXRDY);		//Interrupt reading ready
	NVIC_EnableIRQ(UART0_IRQn);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		myUart1_init:	Initialisiert UART1.
									Konfiguration der verwendeten Pins, Tack im PMC (Power Management) aktivieren,
									Baudrate und Modus einstellen eines Interrupt wenn daten empfangen werden.
 -------------------------------------------------------------------------------------------------------------------- */
 void myUart1_init (void)
 {
 	pio_configure(PINS_UART1_PIO, PINS_UART1_TYPE, PINS_UART1_MASK, PINS_UART1_ATTR);

 	pmc_enable_periph_clk(ID_UART1);

 	const sam_uart_opt_t uart0_settings =
 	{ sysclk_get_cpu_hz(), UART_SERIAL_BAUDRATE, UART_SERIAL_MODE };
 	
 	uart_init(UART1, &uart0_settings);					//Init UART1 and enable Rx and Tx
 	
 	uart_enable_interrupt(UART1, UART_IER_RXRDY);		//Interrupt reading ready
 	NVIC_EnableIRQ(UART1_IRQn);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		uart_write_string:		Schreiben eines Strings am bergebenen Port. 
											Sendet den bergebenen String an den jeweiligen UART-Port.
					Input:	*p_uart			UART-Port (UART0 oder UART1)
							*uart_string	Pointer auf String der gesendet werden soll.
 -------------------------------------------------------------------------------------------------------------------- */
 void uart_write_string(Uart *p_uart, const char *uart_string)
 {
	 while(*uart_string)
	 uart_write_char(p_uart, *uart_string++);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		uart_write_char:		Schreiben ein Zeichen am bergebenen Port. 
											Sendet das bergebene Zeichen an den jeweiligen UART-Port.
					Input:	*p_uart			UART-Port (UART0 oder UART1)
							data			Das Zeichen das gesendet werden soll.
 -------------------------------------------------------------------------------------------------------------------- */
 void uart_write_char(Uart *p_uart, uint8_t data) 
 {
	 while (!(p_uart->UART_SR & UART_SR_TXRDY));
	 uart_write(p_uart, data);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		uart_read_char:					Lesen eines Zeichen am bergebenen Port. 
					Input:	*p_uart					UART-Port (UART0 oder UART1)
	Variablen:		uint8_t received_data_J2		Globale Variable zum Abspeichern der empfangenen Daten an J2.
					uint8_t received_data_J3		Globale Variable zum Abspeichern der empfangenen Daten an J3.
					unsigned char rd_data_J2		Globale Variable zum Abspeichern und Anzeigen auf dem LCD.
					unsigned char rd_data_J3		Globale Variable zum Abspeichern und Anzeigen auf dem LCD.
 -------------------------------------------------------------------------------------------------------------------- */
 void uart_read_char(Uart *p_uart)
 {
	if (p_uart == UART0)
	{
		received_data_J2 = 0;
		if ((p_uart->UART_SR & UART_SR_RXRDY) != 0)	// receiver ready
		{
			received_data_J2 = (uint8_t) p_uart->UART_RHR;
			rd_data_J2 = p_uart->UART_RHR;
		}
	}
	if (p_uart == UART1)
	{
		received_data_J3 = 0;
		if ((p_uart->UART_SR & UART_SR_RXRDY) != 0)	// receiver ready
		{
			received_data_J3 = (uint8_t) p_uart->UART_RHR;
			rd_data_J3 = p_uart->UART_RHR;
		}
	}
 }

 /* --------------------------------------------------------------------------------------------------------------------
	UART0 interrupt Handler		UART0_Handler: Wird ausgefhrt, wenn Daten an J2 empfangen werden.
								Die empfangenen Daten werden direkt zurck an den Sender geschickt, um dem Sender die 
								eingegebenen Daten sichtbar zu machen. Anschlieend werden die Daten abhngig von der
								Auswahl im Men weiter verarbeitet. Entweder eine Menauswahl getroffen, 
								auf dem LCD-Display angezeigt oder an das RFM-Modul bergeben und gesendet.
 -------------------------------------------------------------------------------------------------------------------- */
 void UART0_Handler(void)			// RS232 J2
 {
	/* read and store received data */
	uart_read_char(UART0);

	/* loop back data */
	if (uart_Menu_aktive == false)
	{
		uart_write_char(UART0, received_data_J2);
	}

	/* received data on J2 used to get the user choice in menu */
	if (uart_Menu_aktive == true)
	{
		uart_getMenuChoice(received_data_J2);
	}

	/* send received data on J2 to the LCD */
	if (lcd_showReceivedData == true)
	{
		if (rd_data_J2 == 27)		//ESC
		{
			lcdClearLine(LINE2);
			lcdClearLine(LINE3);
			lcd_setCursor(0, LINE2);
		}
		else
		{
			LCD_writeReceivedData(rd_data_J2);
		}
	}

	/* send received data on J2 via RFM */
	if (uart_sendDataRFM == true)
	{
		buffer[0] = received_data_J2;					// store data in buffer
		uart_write_char(UART0, received_data_J2);		// loop data back
		rfm_sendArray(buffer, MODULE_PAYLOAD);			// send data via rfm
	}
 }

 /* --------------------------------------------------------------------------------------------------------------------
	UART1 interrupt Handler		UART1_Handler: Wird ausgefhrt, wenn Daten an J3 empfangen werden.
								Die empfangenen Daten werden direkt zurck an den Sender geschickt, um dem Sender die 
								eigenen Daten sichtbar zu machen. Anschlieend werden die Daten, wenn aktuell vom 
								Benutzer so im Men ausgewhlt, am LCD angezeigt.
 -------------------------------------------------------------------------------------------------------------------- */
 void UART1_Handler(void)			//RS232 J3
 {
	/* read and store received data */
	uart_read_char(UART1);

	/* loop back data on J3 */
	uart_write_char(UART1, received_data_J3);

	/* check user settings if received data on J3 should be sent via RFM */
	if (uart_rfm_J3_def == true)
	{
		buffer[0] = received_data_J3;
		rfm_sendArray(buffer, 1);
	}

	/* send received data on J3 to the LCD */
	if (lcd_showReceivedData == true)
	{
		if (rd_data_J3 == 27)		//ESC
		{
			lcdClearLine(LINE2);
			lcdClearLine(LINE3);
			lcd_setCursor(1, LINE2);
		}
		else
		{
			LCD_writeReceivedData(rd_data_J3);
		}
	}

	/* send received data on J3 to J2 */
	if (uart_showDataJ3 == true)
	{
		uart_write_char(UART0, received_data_J3);
	}
 }