/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	main.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 *
 *	Dieses Programm wurde zu Testzwecken geschrieben. Es gibt dem Nutzer die Mglichkeit
 *	die verwendete Hardware (ARM-SAMxS-Evaluationboard und Add-On-Module) auf Funktion zu prfen. Des Weiteren
 *	soll es als Inspirationsquelle und als Beispiel dienen. Aus diesem Grund wurde versucht, den Code
 *	sehr ausfhrlich und zustzlich auch auf deutsch zu kommentieren. Darber hinaus wird das ASF benutzt.
 *	Weiter Informationen zum ASF unter http://www.atmel.com/tools/avrsoftwareframework.aspx
 */

#include <asf.h>
#include "user.h"
#include "PES/lcd.h"
#include "PES/timer.h"
#include "PES/buttons_adc.h"
#include "PES/uart.h"
#include "PES/mySpi.h"
#include "PES/enc.h"
#include "PES/dcf.h"
#include "PES/rtc.h"
#include "PES/uart_menu.h"
#include "PES/rfm.h"
#include "conf_usb.h"
#include "user_board.h"
#include "string.h"

static volatile bool main_b_msc_enable = false;

int main(void)
{
	// Initialize and enable Interrupt
	irq_initialize_vectors();
	cpu_irq_enable();
	// Initialize system clock and board
	sysclk_init();
	board_init();
	// Initialize RS232_1 and RS232_2 via UART
	myUart0_init();
	myUart1_init();
	// Initialize the ADC to manage buttons B1-B5
	adc_setup();
	// Configure timer counter TC0 channel 0 and 1
	tc_config_TC0(TIMER_FREQUENZ_0);
	tc_config_TC1(TIMER_FREQUENZ_1);
	// Initialize the SPI for communication with the ENC
	mySpi_init();
	// Initialize the ENC
	enc_init();
	// Initialize the SD-Card interface
	sd_mmc_init();
	// Start USB stack to authorize VBus monitoring
	udc_start();
	// Initialize the LCD and send the welcome screen
	LCD_Init();
	LCD_WriteWelcome();
	// Initialize RTC
	rtc_setup();
	// Initialize RFM
	rfm_init();
	delay_ms(50);

	/* Configure RFM module as transmitter ********************/
	#ifdef RFM_TRANSMITTER
	rfm_configTX(MODULE_TX_NR_0);
	uart_write_string(UART1, "RFM configured as transmitter.\r");
	//uint8_t rfm_text[MODULE_PAYLOAD] = "PollinElectronic";
	//rfm_sendArray(rfm_text, MODULE_PAYLOAD);
	#endif

	/* Configure RFM module as receiver **********************/
	#ifdef RFM_RECEIVER
	rfm_configRX();
	uart_write_string(UART1, "RFM configured as receiver.\r");
	#endif

	// Show the menu via UART
	show_UART_MainMenu();

	// main loop only manages the usb connection 
	while (1) 
	{	
		if (main_b_msc_enable)
		{
			udi_msc_process_trans();
		}
	}
}

bool main_msc_enable(void)
{
	main_b_msc_enable = true;
	return true;
}

void main_msc_disable(void)
{
	main_b_msc_enable = false;
}